function rxFSK = demodulateFSK(packetWaveform, numberOfBytes, fs, fc, symbolsPerSecond, receiverType, useLimiter)
    useQuadrature = 0;
    c = strcmp(receiverType, 'quad');
    if (c == 1)
        useQuadrature = 1;
    else
        c = strcmp(receiverType, 'env');
        if (c ~= 1)
            s = sprintf('Error: Unknown receiver type %s.', receiverType);
            disp(s);
            rxFSK = -1;
            return;
        end
    end
    samplesPerBit = floor(fs / symbolsPerSecond);
    carrierFreq = fc;
    fc = [(carrierFreq - symbolsPerSecond / 2) (carrierFreq + symbolsPerSecond / 2)];

    if (symbolsPerSecond < 500)
        chirpRange = [(carrierFreq - 500) (carrierFreq + 500)];
    else
        chirpRange = [(carrierFreq - symbolsPerSecond) (carrierFreq + symbolsPerSecond)];
    end
    packetChirp = chirp(chirpRange(1), chirpRange(2), fs, 0.05);
    guardTime = zeros(1, 0.01 * fs);
    bitsInByte = 8;
    numberOfBits = numberOfBytes * bitsInByte;
    
    %% Synchronize on packet chirp signals
    prefixLength = length(packetChirp) + length(guardTime);
    correlatedSamples = real(fftshift(xcorr(packetWaveform(1:prefixLength), conj(packetChirp))));
    [~, maxIndex] = max(correlatedSamples);
    len = length(correlatedSamples);
    if (maxIndex > len/2)
        maxIndex = maxIndex - len;
    end
    start = maxIndex + prefixLength;
    packetWaveform = packetWaveform(start:start+samplesPerBit * numberOfBits);

    %% Demodulation
    bitStream = zeros(1, numberOfBits);

    % Demodulate with quadrature receiver
    if (useQuadrature) 
        % Create reference signals
        t = 0:samplesPerBit-1;
        cos0 = cos(2 * pi * fc(1)/fs * t);
        sin0 = sin(2 * pi * fc(1)/fs * t);
        cos1 = cos(2 * pi * fc(2)/fs * t);
        sin1 = sin(2 * pi * fc(2)/fs * t);

        for i = 1:numberOfBits
            % Quadrature (energy) detector
            if (useLimiter)
                rcv = hardlimit(packetWaveform((i-1)*samplesPerBit + 1:i*samplesPerBit));
            else
                rcv = packetWaveform((i-1)*samplesPerBit + 1:i*samplesPerBit);
            end
            Izero = rcv .* cos0;
            Qzero = rcv .* sin0;
            Ione = rcv .* cos1;
            Qone = rcv .* sin1;
            z1 = (sum(Izero))^2;
            z2 = (sum(Qzero))^2;
            z3 = (sum(Ione))^2;
            z4 = (sum(Qone))^2;
            energy0 = z1 + z2;
            energy1 = z3 + z4;
            bitStream(i) = (energy1 > energy0);
        end
    % Demodulate with envelope detector
    else
        % Prepare FSK signal for envelope detector
        if (useLimiter)
            zeroSignal = abs(hilbert(filterSignal(hardlimit(packetWaveform), fs, fc(1), symbolsPerSecond)));
            oneSignal = abs(hilbert(filterSignal(hardlimit(packetWaveform), fs, fc(2), symbolsPerSecond)));
        else
            zeroSignal = abs(hilbert(filterSignal(packetWaveform, fs, fc(1), symbolsPerSecond)));
            oneSignal = abs(hilbert(filterSignal(packetWaveform, fs, fc(2), symbolsPerSecond)));
            maxZero = max(zeroSignal);
            maxOne = max(oneSignal);
            if (maxZero > maxOne)
                oneSignal = oneSignal * (maxZero/maxOne);
            else
                zeroSignal = zeroSignal * (maxOne/maxZero);
            end
        end
        diff = oneSignal - zeroSignal;

        for i = 1:numberOfBits
            % Envelope detector for FSK demodulation (sample in second half of symbol)
            rcv = sum(diff((i-1)*samplesPerBit + floor(samplesPerBit/2):i*samplesPerBit));
            bitStream(i) = (rcv > 0);
        end
    end
    
    %% Convert back into chars
    numberOfBytes = numberOfBits / bitsInByte;
    byteStream = zeros(1, numberOfBytes);
    for i = 1:numberOfBytes
        byte = uint8(0);
        for j = 1:bitsInByte
            bit = uint8(bitand(bitStream((i - 1) * bitsInByte + j), 1));
            byte = bitor(byte, bit);
            if (j < bitsInByte)
                byte = bitshift(byte, 1);
            end
        end
        byteStream(i) = byte;
    end

    rxFSK = byteStream;