function [y] = fconv(x, h, normalize)
%FCONV Fast Convolution
%   y = FCONV(x, h) convolves x and h, where
%       x is the time domain signal and
%       h is the filter kernel.
%
%   y = FCONV(x, h, normalize) convolves x and h and normalizes the
%       result to -1..1 if normalize is 'true'.

    if (nargin < 2)
        error('Too few arguments specified in function.');
    end

    if (nargin > 3)
        error('Too many arguments specified in function.');
    end

    if (nargin == 2)
        normalize = 'false';
    end

    lenY = length(x) + length(h) - 1;
    lenYPow2 = pow2(nextpow2(lenY)); % Find smallest power of 2 > lenY
    X = fft(x, lenYPow2);            % FFT
    H = fft(h, lenYPow2);            % FFT
    Y = X .* H;                      % Multiplication in frequency domain
    y = real(ifft(Y, lenYPow2));     % Inverse FFT
    y = y(1:lenY);                   % Use first lenY elements
    if (strcmp(normalize, 'true') == 1)
        y = y / max(abs(y));         % Normalize output
    end
