function [y] = filterSignal(x, samplingRate, freq, symbolsPerSecond)
%FILTERSIGNAL implements a second-order IIR bandpass filter.
%   y = FILTERSIGNAL(x, samplingRate, freq, symbolsPerSecond)
%       produces a bandpass-filtered signal of the input signal x with a
%       -3 dB bandwidth of symbolsPerSecond Hz centered on freq Hz.
%       The samplingRate of x must be provided as an input argument.

    f = freq / samplingRate;
    bw = symbolsPerSecond / samplingRate;
    R = 1 - 3 * bw;
    K = (1 - 2 * R * cos(2 * pi * f) + R * R) / (2 - 2 * cos(2 * pi * f));
    a0 = 1 - K;
    a1 = 2 * (K - R) * cos(2 * pi * f);
    a2 = R * R - K;
    b1 = 2 * R * cos(2 * pi * f);
    b2 = -R * R;

    numSamples = length(x);
    y = zeros(1, numSamples);
    y(1) = 0;
    y(2) = 0;
    for i = 3:numSamples
        y(i) = a0 * x(i) + a1 * x(i - 1) + a2 * x(i - 2) + ...
               b1 * y(i - 1) + b2 * y(i - 2);
    end