function [txFSK numberOfSamples] = modulateFSK(byteStream, fs, fc, symbolsPerSecond)
    numberOfBytes = length(byteStream);
    bitsInByte = 8;
    numberOfBits = numberOfBytes * bitsInByte;
    bitStream = zeros(1, numberOfBits);
    for i = 1:numberOfBytes
        byte = uint8(byteStream(i));
        for j = 1:bitsInByte
            bit = bitshift(bitand(byte, 128), -7);
            bitStream((i - 1) * bitsInByte + j) = bit;
            byte = bitshift(byte, 1);
        end
    end

    samplesPerBit = floor(fs / symbolsPerSecond);
    carrierFreq = fc;
    fc = [(carrierFreq - symbolsPerSecond / 2) (carrierFreq + symbolsPerSecond / 2)];

    if (symbolsPerSecond < 500)
        chirpRange = [(carrierFreq - 500) (carrierFreq + 500)];
    else
        chirpRange = [(carrierFreq - symbolsPerSecond) (carrierFreq + symbolsPerSecond)];
    end
    packetChirp = chirp(chirpRange(1), chirpRange(2), fs, 0.05);
    guardTime = zeros(1, 0.01 * fs);

    tx = zeros(1, samplesPerBit * numberOfBits);
    t = 0.0;
    for i = 1:numberOfBits
        for j = 1:samplesPerBit
            t = t + 2 * pi * fc(bitStream(i)+1) / fs;
            if (t > pi)
                t = t - 2 * pi;
            end
            tx(j + samplesPerBit*(i-1)) = cos(t);
        end
    end

    txFSK = [guardTime real(packetChirp) guardTime tx];
    numberOfSamples = length(txFSK);
