//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//

#include "App.h"
#include "Datagram.h"
#include <random.h>
#include <omnetpp.h>
#include <string>
#include <fstream>

// The module class needs to be registered with OMNeT++
Define_Module(App);

App::App() {
    generateDatagram = NULL;
	datagramNumber = 0;
	ifstream ifs("data/PacketData.txt");
	string temp;

	while (getline(ifs, temp)) {
	    data.push_back(temp);
	}
}

App::~App() {
    cancelAndDelete(generateDatagram);
    data.clear();
}

void App::initialize() {
	address = static_cast<unsigned short>(par("address"));
	totalDatagramsToSend = par("totalDatagramsToSend");
    sendIATime = &par("sendIATime");  // volatile parameter
    if (address == 0) {
        generateDatagram = new cMessage("nextDatagram");
        scheduleAt(sendIATime->doubleValue(), generateDatagram);
        if (ev.isGUI()) getParentModule()->bubble("Generating datagram...");
    }
}

void App::handleMessage(cMessage *msg) {
    if (msg == generateDatagram) {
        datagramNumber++;
        char *buf = new char[128];
        sprintf(buf, "Datagram%d", datagramNumber);
        Datagram *datagram = new Datagram(buf);

        messageID = (int)intrand(data.size());

        cout << endl << "Generating datagram " << datagramNumber << "..." << endl;
        datagram->setMessageID(messageID);
        datagram->setPayload(data.at(messageID));
        datagram->setSourceAddress(address);
        datagram->setDestinationAddress(1-address);
        datagram->constructByteStream();
        send(datagram, "outToLink");

        if (datagramNumber < totalDatagramsToSend) {
            scheduleAt(simTime() + sendIATime->doubleValue(), generateDatagram);
            if (ev.isGUI()) getParentModule()->bubble("Generating datagram...");
        }
    } else {
        EV << "Received message at node " << address << "." << endl;
        Datagram *datagram = check_and_cast<Datagram *>(msg);
        cout << "Expected payload: " << datagram->getPayload() << endl;
        cout << "Actual payload  : ";
        datagram->displayDemodulatedMessage();
        delete datagram;
        if (ev.isGUI()) {
            getParentModule()->getDisplayString().setTagArg("i", 1, "green");
            getParentModule()->bubble("Arrived!");
        }
    }
}
