//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//

#include "Link.h"
#include "Datagram.h"

Define_Module(Link);

Link::Link() { }

Link::~Link() { }

void Link::initialize() {
	address = par("address");
}

void Link::handleMessage(cMessage *msg) {
	cGate* gate = msg->getArrivalGate();
	Datagram *datagram = check_and_cast<Datagram *>(msg);
	EV << "Arrived at link layer on " << gate->getBaseName() << endl;
	if (datagram->arrivedOn("inFromApp")) {
	    unsigned short checksum = datagram->computeChecksum();
        EV << "Setting checksum: " << checksum << endl;
        datagram->setChecksum(checksum);
        cout << "Bits sent:" << endl;
        datagram->displayMessage(true);
		send(datagram, "outToPhy");
	} else {
	    cout << "Bits received:" << endl;
	    datagram->displayMessage(false);
	    unsigned short storedChecksum = datagram->getChecksum();
	    unsigned short computedChecksum = datagram->computeChecksum();
	    if (storedChecksum != computedChecksum) {
	        EV << "Stored checksum " << storedChecksum << " does not match computed checksum " << computedChecksum << "." << endl;
	        EV << "Datagram discarded." << endl;
	        delete datagram;
	    } else {
            EV << "Stored checksum " << storedChecksum << " matches computed checksum " << computedChecksum << "." << endl;
            if (datagram->getDestinationAddress() == address) {
                send(datagram, "outToApp");
            }
	    }
	}
}
