//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//

#ifndef DATAGRAM_H_
#define DATAGRAM_H_

#include <cmessage.h>
#include <string>

using namespace std;

class Datagram: public cPacket {
    public:
        Datagram(const cPacket &packet);
        Datagram(const char *name=NULL, short kind=0, int64 length=0);
        virtual ~Datagram();
        void setPayload(string dataString);
        void setSourceAddress(unsigned short sourceAddress);
        void setDestinationAddress(unsigned short destinationAddress);
        void setMessageID(int messageID);
        void setModulationType(const char* modulationTypeStr);
        void setSamplingRate(int samplingRate);
        void setCarrierFrequency(int carrierFrequency);
        void setSymbolsPerSecond(int symbolsPerSecond);
        void setSignalLevelDB(double signalLevelDB);
        void setChecksum(unsigned short checksum);
        void setModulatedSamples(int length, double *samples);
        void setDemodulatedByteStream(int length, double *bytes);  // MATLAB interface return doubles instead of unsigned chars
        void setConvolvedSamples(int length, double *samples);
        int getLength();
        string getPayload();
        unsigned short getSourceAddress();
        unsigned short getDestinationAddress();
        int getMessageID();
        int getModulationType();
        const char* getModulationTypeStr();
        int getSamplingRate();
        int getCarrierFrequency();
        int getSymbolsPerSecond();
        double getTransmissionDelayInSeconds();
        double getSignalLevelDB();
        unsigned short getChecksum();
        unsigned char *getByteStream();
        int getByteStreamLength();
        double *getModulatedSamples();
        int getModulatedSamplesLength();
        unsigned char *getDemodulatedByteStream();
        int getDemodulatedByteStreamLength();
        double *getConvolvedSamples();
        int getConvolvedSamplesLength();
        unsigned short computeChecksum();
        double computeBER();
        void constructByteStream();
        void displayMessage(bool useExpectedResults);
        void displayDemodulatedMessage();

        const static int headerLengthInBytes = 8;
        const static int MOD_FSK = 0, MOD_PSK = 1;
    private:
        string payload;
        const char *modulationTypeStr;
        unsigned short sourceAddress;      // source address
        unsigned short destinationAddress; // destination address
        int messageID;                     // ID of payload from finite list
        int samplingRate;
        int carrierFrequency;
        int symbolsPerSecond;
        int numberOfBytes;
        int numberOfModulatedSamples;
        int numberOfDemodulatedBytes;
        int numberOfConvolvedSamples;
        short crc;
        double signalLevelDB;
        double *modulatedSamples;
        double *convolvedSamples;
        unsigned char *byteStream;
        unsigned char *demodulatedByteStream;
};

#endif /* DATAGRAM_H_ */
